#include "Dal.h"
#include <iostream>
#include "SasDalUtils.h"
#include "dal_utilities.h"
#include "DataSubspace.h"
#include<cstdlib>
#include<vector>
#include<cmath>


#include <f90cxx.h>

/****************************************************
*
*
*
*****************************************************/
struct OmSourceParameter {
    OmSourceParameter() :
    
    number(0), 
    xcoord(0), 
    ycoord(0),
    rightascension(0), 
    declination(0),  
    positionerror(0),
    exposure(0), 
    significance(0),
    rawcountrate(0), 
    rawcountrateerror(0),
    corrcountrate(0),
    corrcountrateerror(0),  //
    magnitude(0),
    magnitudeerror(0),  
    rawbackgroundcountrate(0),
    rawbackgroundcountrateerror(0),
    corrbackgroundcountrate(0),
    corrbackgroundcountrateerror(0),
    psf1correction(0),
    psf2correction(0),
    coincidencecorrection(0), 
    tdscorr(0),
    semia(0),
    semiaerr(0),
    semib(0),
    semiberr(0),
    theta(0),  
    thetaerr(0),
    photometryradius(0),   // was 0
    axialRatio(1.0),
    alpha(0.0), 
    innerbackgroundradius(0),
    outerbackgroundradius(0),
    nosourcepixels(0),
    extensionflag(0),
    confusionflag(0), 
    qualityflag(0),
    iMin(0), iMax(0), jMin(0), jMax(0) 

  {}   

    int number; 
    float xcoord; 
    float ycoord;
    double rightascension; 
    double declination;  
    float positionerror;
    float exposure; 
    float significance;
    float rawcountrate; 
    float rawcountrateerror;
    float corrcountrate;
    float corrcountrateerror;
    float magnitude;
    float magnitudeerror;  
    float rawbackgroundcountrate;
    float rawbackgroundcountrateerror;
    float corrbackgroundcountrate;
    float corrbackgroundcountrateerror;
    float psf1correction;
    float psf2correction;
    float coincidencecorrection; 
    float tdscorr;
    float semia, semiaerr;
    float semib, semiberr;
    float theta, thetaerr;  
    float photometryradius;
    float axialRatio;
    float alpha;   
    float innerbackgroundradius;
    float outerbackgroundradius;
    int nosourcepixels;
    Integer1 extensionflag;
    Integer1 confusionflag; 
    Integer2 qualityflag;
    Integer2 iMin; 
    Integer2 iMax; 
    Integer2 jMin; 
    Integer2 jMax; 

};



bool createSrcListFile(const string newSrcFile, vector< OmSourceParameter > & parameters);



void addReal32Image(DataSet *set, const char *name, const vector< vector<real32> >& image);

void addInt8Image(DataSet *set, const char *name, const vector< vector<int8> >& image);

void addInt16Image(DataSet *set, const char *name, const vector< vector<int8> >& image);



/******************************************************
*
*
*
*
*
*******************************************************/
bool LoadVectorFloatColumnData(const Table *table, 
                      const string& columnName, 
		      int &size, vector<float> &data);

bool LoadVectorDoubleColumnData(const Table *table, 
                      const string& columnName, 
		      int &size, vector<double> &data);
void loadData(string file, vector<double> &ra, vector<double> &dec);







bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
	             vector<real32> &data);

bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
	             vector<real64> &data);

bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
	             vector<int8> &data);

bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
	             vector<int16> &data);


bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
	             vector<int32> &data);




void setColumnData( const int size, const ColumnData *columnData, const vector<int8>& data);
void setColumnData( const int size, const ColumnData *columnData, const vector<int16>& data);
void setColumnData( const int size, const ColumnData *columnData, const vector<int32>& data);
void setColumnData( const int size, const ColumnData *columnData, const vector<real32>& data);
void setColumnData( const int size, const ColumnData *columnData, const vector<real64>& data);



void createImageFile(string file, int nx, int ny);

 
/**********************************************************************
*
*
* First test is to create a list of sources and then add the quality flags 
* to the sky image
*
*
**********************************************************************/ 

int main()
{


    srand ( time(NULL) );
    int number = max(10, rand() % 1000);

    //    number = 10;
    vector<OmSourceParameter> source(number); 


    int nx = max(100, rand() % 500 + 1);
    int ny = max(100, rand() % 500 + 1);

    nx = ny = 1024;
    cout << "nx=" << nx << " ny=" << ny << endl;



    createImageFile("testimage.fits", nx, ny);


    source[0].xcoord = float(nx / 2 + 1);
    source[0].ycoord = float(ny / 2 + 1);
    source[0].rawcountrate = 80.0;

    source[1].xcoord = float(nx / 4 + 1);
    source[1].ycoord = float(ny / 2 + 1);
    source[1].rawcountrate = 80.0;


    source[2].xcoord = 0.75 * float(nx + 1);
    source[2].ycoord = float(ny / 2+ 1);
    source[2].rawcountrate = 80.0;



    source[3].xcoord = 0.75 * float(nx + 1);
    source[3].ycoord = float(ny / 4+ 1);
    source[3].rawcountrate = 80.0;



    source[4].xcoord = float(nx / 4 + 1);
    source[4].ycoord = float(ny / 4 + 1);
    source[4].rawcountrate = 80.0;







    for(int i = 5; i < number; i++)
    {
      //  if(i == 4) continue;
        source[i].xcoord = float(rand() % nx + 1);   
        source[i].ycoord = float(rand() % ny + 1);           
	source[i].rawcountrate = float(rand() % 100 + 1);   
	// source[i].rawcountrate = 10.0;
	//	cout << i << ") x=" << source[i].xcoord << " y=" << source[i].ycoord << " rawrate=" << source[i].rawcountrate << endl;

    }


    if (!createSrcListFile( "test.fits", source) )
    { 


    }

    return 0;


}

/***********************************************************
*
*
*
*
**********************************************************/
void loadData(string file, vector<double> &ra, vector<double> &dec)
{

    DataSet *dataSet = dataSetServer->open( file, Dal::Read);

    if (!dataSet)
    {


    }    
 
    const Table *table = dataSet->table( "SRCLIST", Dal::Read);
    if( !table) {};

    /**********************************************************************
    *
    *
    **********************************************************************/
  
    int size = 0;

 
    if ( !LoadVectorDoubleColumnData( table, "RA_CORR", size, ra) )
    {

    

    }

 
    if ( !LoadVectorDoubleColumnData( table, "DEC_CORR", size, dec) )
    {

    

    }

}
/********************************************************
*
*
******************************************************/
bool LoadVectorFloatColumnData(const Table *table, 
                      const string& columnName, 
                      int &size, vector<float> &data)
{
     
    size = table->rows();
   
    if (table->hasColumn(columnName))
    { 

        const ColumnData *columnData = table->column(columnName)->data();
        if (!columnData) exit(0);
	

      
        float *xData = columnData->real32Data();
       


	for (int i=0; i< size; i++) 
        {
	  data.push_back( xData[i] ); 
	 
	}
     }
         
   
    return true;

};

/********************************************************
*
*
******************************************************/
bool LoadVectorDoubleColumnData(const Table *table, 
                      const string& columnName, 
                      int &size, vector<double> &data)
{
       
    size = table->rows();
   
    if (table->hasColumn(columnName))
    { 
        const ColumnData *columnData = table->column(columnName)->data();
        if (!columnData) exit(0);
	

      
        double *xData = columnData->real64Data();
       


	for (int i=0; i< size; i++) 
        {
	  data.push_back( xData[i] ); 
	 
	}
     }
         
   
    return true;
};



/********************************************************
*
*
******************************************************/
bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
                     vector<real32> &data)
{
     

  
    // Get the size of the table
    vector<unsigned long> size(1);
    size[0] = 1;
    size[1] = nData;
  
   
    // Create the column
    Column *column = table->addColumn( columnName, Dal::Real32, comment, units , size);
 
    ColumnData *columnData = column->data();

    setColumnData( ( int)size[1], columnData, data);


    delete columnData;

          
    return true;

};



/********************************************************
*
*
******************************************************/
bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
                     vector<real64> &data)
{
     

  
    // Get the size of the table
    vector<unsigned long> size(1);
    size[0] = 1;
    size[1] = nData;
  
   
    // Create the column
    Column *column = table->addColumn( columnName, Dal::Real64, comment, units , size);
 
    ColumnData *columnData = column->data();

    setColumnData( ( int)size[1], columnData, data);


    delete columnData;


          
    return true;
};

/********************************************************
*
*
******************************************************/
bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
                     vector<int8> &data)
{
     

  
    // Get the size of the table
    vector<unsigned long> size(1);
    size[0] = 1;
    size[1] = nData;
  
   
    // Create the column
    Column *column = table->addColumn( columnName, Dal::Int8, comment, units , size);
 
    ColumnData *columnData = column->data();

    setColumnData( ( int)size[1], columnData, data);


    delete columnData;


          
    return true;
};


/********************************************************
*
*
******************************************************/
bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
                     vector<int16> &data)
{
     

  
    // Get the size of the table
    vector<unsigned long> size(1);
    size[0] = 1;
    size[1] = nData;
  
   
    // Create the column
    Column *column = table->addColumn( columnName, Dal::Int16, comment, units , size);
 
    ColumnData *columnData = column->data();

    setColumnData( ( int)size[1], columnData, data);


    delete columnData;


          
    return true;
};



/********************************************************
*
*
*
*
*
******************************************************/
bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
                     vector<int32> &data)
{
     

  
    // Get the size of the table
    vector<unsigned long> size(1);
    size[0] = 1;
    size[1] = nData;
  
   
    // Create the column
    Column *column = table->addColumn( columnName, Dal::Int32, comment, units , size);
 
    ColumnData *columnData = column->data();

    setColumnData( ( int)size[1], columnData, data);

    delete columnData;
          
    return true;
};


/******************************************************
*
*
*
******************************************************/
void setColumnData( const int size, const ColumnData *columnData, const vector<int8>& data)
{
    int8 * r64 = columnData->int8Data();
    for(int i = 0; i < size; i++)
        r64[ i] = data[i];


}




/******************************************************
*
*
*
******************************************************/
void setColumnData( const int size, const ColumnData *columnData, const vector<int16>& data)
{
    int16 * r64 = columnData->int16Data();
    for(int i = 0; i < size; i++)
        r64[ i] = data[i];


}


/******************************************************
*
*
*
******************************************************/
void setColumnData( const int size, const ColumnData *columnData, const vector<int32>& data)
{
    int32 * r64 = columnData->int32Data();
    for(int i = 0; i < size; i++)
        r64[ i] = data[i];


}








/******************************************************
*
******************************************************/


void setColumnData( const int size, const ColumnData *columnData, const vector<real32>& data )
{
    real32 * r32 = columnData->real32Data();
    for(int i = 0; i < size; i++)
        r32[i] = data[i];

}

/******************************************************
*
******************************************************/
void setColumnData( const int size, const ColumnData *columnData, const vector<real64>& data)
{
    real64 * r64 = columnData->real64Data();
    for(int i = 0; i < size; i++)
        r64[ i] = data[i];


}





 
/**********************************************************************
*
*********************************************************************/
bool createSrcListFile(const string newSrcFile, vector< OmSourceParameter > & parameters)
{ 
                     
    DataSet *dataSet = dataSetServer->open(newSrcFile, Dal::Create);
  
    if ( !dataSet ) return false;
   

   *dataSet->addAttribute( "FILTER", "" ) = "V";

   *dataSet->addAttribute( "OBS_ID", "" ) = "666";

   *dataSet->addAttribute( "EXPOSURE", "" ) = 1000.0;

   *dataSet->addAttribute( "WINDOWX0", "" ) = 0;
   *dataSet->addAttribute( "WINDOWDX", "" ) = 0;


   *dataSet->addAttribute( "WINDOWY0", "" ) = 0;
   *dataSet->addAttribute( "WINDOWDY", "" ) = 0;


   *dataSet->addAttribute( "BINAX1", "" ) = 1;
   *dataSet->addAttribute( "BINAX2", "" ) = 1;

   *dataSet->addAttribute( "BINBPE", "" ) = false;



    // Add the table
    Table *table = dataSet->addTable( "SRCLIST", parameters.size() );


   cout << "line 2128\n";

    vector<real32> data;
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].xcoord );


    if ( !addColumn( table, "XPOS", "pixels", "x coordinate",   
			   (int)parameters.size(), data) )
      return false;



    data.clear();
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].ycoord );

    if ( !addColumn( table, "YPOS", "pixels", "y coordinate",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();


    vector<real64> data1;
    for (unsigned i = 0; i < parameters.size(); i++)
        data1.push_back( parameters[i].rightascension );


    if ( !addColumn( table, "RA", "degrees", "Right-ascension",   
			   (int)parameters.size(), data1) )
      return false;

    data1.clear();
    for (unsigned i = 0; i < parameters.size(); i++)
        data1.push_back( parameters[i].declination );


    if ( !addColumn( table, "DEC", "degrees", "Declination",   
			   (int)parameters.size(), data1) )
      return false;

    data1.clear();


    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].positionerror );


    if ( !addColumn( table, "POSERR", "arc secs", "Position error",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

    /**********************************************************************
     * Count rates
     ********************************************************************/

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].rawcountrate );


    if ( !addColumn( table, "RATE", "counts/sec", "Raw count-rate",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].rawcountrateerror );


    if ( !addColumn( table, "RATE_ERR", "counts/sec", "Raw count-rate error",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].rawbackgroundcountrate );


    if ( !addColumn( table, "BACKGROUND_RATE", "counts/sec", "Raw background count-rate",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].significance );


    if ( !addColumn( table, "SIGNIFICANCE", "", "Signa-to-noise",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].corrcountrate );


    if ( !addColumn( table, "CORR_RATE", "counts/sec", "Corrected count-rate",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].corrcountrateerror );


    if ( !addColumn( table, "CORR_RATE_ERR", "counts/sec", "Corrected count-rate error",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].corrbackgroundcountrate );


    if ( !addColumn( table, "CORR_BACKGROUND_RATE", "counts/sec", "Corrected background count-rate",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 
    /*****************************************************************************
     * Corrections applied
     ****************************************************************************/ 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].psf1correction );


    if ( !addColumn( table, "PSF1_CORR", "", "PSF1 correction",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].psf2correction );


    if ( !addColumn( table, "PSF2_CORR", "", "UV PSF2 correction",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

    /*************************************************************************************
     * Instrumental magnitudes
     ************************************************************************************/
    
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].magnitude);


    if ( !addColumn( table, "MAG", "mag", "Instrumental magnitude",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
     
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].magnitudeerror);


    if ( !addColumn( table, "MAGERR", "mag", "Instrumental magnitude error",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

    vector<int> data3;
    for (unsigned i = 0; i < parameters.size(); i++)
        data3.push_back( parameters[i].nosourcepixels);


    if ( !addColumn( table, "NOPIXELS", "", "No of source pixels (extended sources)",   
			   (int)parameters.size(), data3) )
      return false;


    data3.clear();    

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].photometryradius);


    if ( !addColumn( table, "APERTURE", "unbinned pixels", "Extraction aperture radius (point-sources)",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
     
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].semia);


    if ( !addColumn( table, "FWHM_MAJ", "arcsec", "FWHM major-axis of source",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 
    
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].semiaerr);


    if ( !addColumn( table, "FWHM_MAJ_ERR", "arcsecs", "FWHM major-axis error of source",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 
    
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].semib);


    if ( !addColumn( table, "FWHM_MIN", "arcsecs", "FWHM minor-axis of source",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 
    
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].semiberr);


    if ( !addColumn( table, "FWHM_MIN_ERR", "arcsecs", "FWHM minor-axis error of source",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 
    
 
    
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].theta);


    if ( !addColumn( table, "PA", "counts/sec", "Position angle of major-axis of source",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

 
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].thetaerr);


    if ( !addColumn( table, "PA_ERR", "counts/sec", "Error in position-angel of major-axis of source",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 




    vector<int8> data4;
 
    for (unsigned i = 0; i < parameters.size(); i++)
        data4.push_back( parameters[i].extensionflag);


    if ( !addColumn( table, "EFLAG", "", "Source extension flag (0-point, 1-extended)",   
			   (int)parameters.size(), data4) )
      return false;


    data4.clear();


    vector<int16> data5;
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data5.push_back( parameters[i].qualityflag);


    if ( !addColumn( table, "QFLAG", "", "Quality flag",   
			   (int)parameters.size(), data5) )
      return false;


    data4.clear();
 
    for (unsigned i = 0; i < parameters.size(); i++)
        data4.push_back( parameters[i].confusionflag);


    if ( !addColumn( table, "CFLAG", "", "Confusion flag",   
			   (int)parameters.size(), data4) )
      return false;


    data4.clear();
 

    //    table->insertRows( (int)parameters.size(), 100);

    dataSetServer->close( dataSet );

   
    return true;
}


/*******************************************************
*
*
*******************************************************/
void addInt8Image(DataSet *set, const char *name, const vector< vector<int8> >& image)

{


    unsigned nx = image.size();
    unsigned ny = image[0].size();
    longVec size(2);
    size[0] = ny;
    size[1] = nx;
   
   
    cout << "Adding integer 8 quality image\n";
  
  
    Array * array  = set->addArray(name,
				   Dal::Int8,
				   size);
    ArrayData * arraydata = array->data();
    int8 * data = arraydata->int8Data();
 

    array->nullValue((long)127);

    cout << "nullvalue=" << array->intNullValue() << endl;

    unsigned dx = size[1];
    unsigned dy = size[0];

    for (unsigned x = 0; x < dx; ++x)
    {
      for (unsigned y = 0; y < dy; ++y)
      {
             
	  int index = x + y * dx; 
	  if( image[x][y] < 127)
	  {    
	        data[index] = image[x][y];
	  }
	  else
	  {
	      arraydata->setToNull(index);
          }
      }

    }

    delete arraydata;


}



/*******************************************************
*
*
*******************************************************/
void addInt16Image(DataSet *set, const char *name, const vector< vector<int16> >& image)

{


    unsigned nx = image.size();
    unsigned ny = image[0].size();
    longVec size(2);
    size[0] = ny;
    size[1] = nx;
   
   
    cout << "Adding integer 16 quality image\n";
  
  
    Array * array  = set->addArray(name,
				   Dal::Int16,
				   size);
    ArrayData * arraydata = array->data();
    int16 * data = arraydata->int16Data();
 

    array->nullValue((long)32767);

    cout << "nullvalue=" << array->intNullValue() << endl;

    unsigned dx = size[1];
    unsigned dy = size[0];

    for (unsigned x = 0; x < dx; ++x)
    {
      for (unsigned y = 0; y < dy; ++y)
      {
             
	  int index = x + y * dx; 
	  if( image[x][y] >= 0)
	  {    
	        data[index] = image[x][y];
	  }
	  else
	  {
	      arraydata->setToNull(index);
          }
      }

    }

    delete arraydata;


}

/*******************************************************
*
*
*******************************************************/
void addReal32Image(DataSet *set, const char *name, const vector< vector<real32> >& image)

{


    unsigned nx = image.size();
    unsigned ny = image[0].size();
    longVec size(2);
    size[0] = ny;
    size[1] = nx;
   
   
 
  
  
    Array * array  = set->addArray(name,
				   Dal::Real32,
				   size);
    ArrayData * arraydata = array->data();
    real32 * data = arraydata->real32Data();
 


    unsigned dx = size[1];
    unsigned dy = size[0];

    for (unsigned x = 0; x < dx; ++x)
    {
      for (unsigned y = 0; y < dy; ++y)
      {
             
	  int index = x + y * dx; 
	  if( image[x][y] >= 0)
	  {    
	        data[index] = image[x][y];
	  }
	  else
	  {
	      arraydata->setToNull(index);
          }
      }

    }

    delete arraydata;


}





/*******************************************************
*
*
*******************************************************/


vector<vector<int16> >  loadInt16Image(const string inSet, const char * name, int &nx, int &ny)
{


    DataSet *set = dataSetServer->open(inSet, Dal::Read);
 


    /*****************************************************************
    * Get the size of the image
    *****************************************************************/
    const Array* array = set->array(name);
 

    const ArrayData * arraydata = array->data();
    const int16 * theData = arraydata->int16Data();
 

    /*****************************************************************
    * Get the size of the image
    *****************************************************************/
 
    longVec size = array->size();
   



    nx = (int)size[1];
    ny = (int)size[0];


    vector< vector<int16> > image(nx); 


    for (int i = 0; i < nx; i++)
    {
        image[i].resize(ny); 
        for (int j = 0; j < ny; j++)
        {
            

	  image[i][j] = theData[i + (j * nx)];  
	 
	
        }

  
    }


    delete arraydata;
   
    dataSetServer->close( set ); // Finished with the file

    return image;
}
/************************************************************
*
*
*
************************************************************/
void createImageFile(string file, int nx, int ny)
{

 
    DataSet *dataSet = dataSetServer->open(file, Dal::Create);







   *dataSet->addAttribute( "FILTER", "" ) = "V";

   *dataSet->addAttribute( "OBS_ID", "" ) = "666";

   *dataSet->addAttribute( "EXPOSURE", "" ) = 1000.0;

   *dataSet->addAttribute( "WINDOWX0", "" ) = 0;
   *dataSet->addAttribute( "WINDOWDX", "" ) = 0;


   *dataSet->addAttribute( "WINDOWY0", "" ) = 0;
   *dataSet->addAttribute( "WINDOWDY", "" ) = 0;


   *dataSet->addAttribute( "BINAX1", "" ) = 1;
   *dataSet->addAttribute( "BINAX2", "" ) = 1;

   *dataSet->addAttribute( "BINBPE", "" ) = false;

   *dataSet->addAttribute( "CRPIX1", "" ) = 0;
   *dataSet->addAttribute( "CRPIX2", "" ) = 0;

   *dataSet->addAttribute( "CRVAL1", "" ) = 45;
   *dataSet->addAttribute( "CRVAL2", "" ) = 45;
   *dataSet->addAttribute( "CDELT1", "" ) = -1.0;
   *dataSet->addAttribute( "CDELT2", "" ) = 1.0;



    vector< vector<float> > image(nx, vector<float>(ny, 0.0));
    addReal32Image( dataSet, "PRIMARY", image);



    vector< vector<int16> > quality(nx, vector<int16>(ny, 0));

    addInt16Image( dataSet, "QUALITY", quality);




    dataSetServer->close( dataSet );



}
